#include "soaring.h"
#include "util.h"
#include "form.h"
#include <Pilot.h>	// all the system toolbox headers
#include <SerialMgr.h>
#include <MemoryMgr.h>

#define GPVTG_VALID 1
#define GPRMC_VALID 2
#define GPBWC_VALID 4
#define GPRMB_VALID 8
#define GPGGA_VALID 16
#define PGRMZ_VALID 32
#define USRALT_VALID 64

void parser_standard(char * serinp, int length)
{

	static int state = 0;
	static int next = 0;
	static int gpsver2 = 0;
	static int revert;
	static char buf[80];
	static char GPVTG[80];  
	static char GPBWC[80]; 
	static char GPRMB[80]; 
	static char GPRMC[80]; 
	static char PGRMZ[80]; 
	static char GPGGA[80]; 
	static char USRALT[80];
	CharPtr s;
	int cur = 0;
	double true_track;
	double magnetic_track;
	double groundspeed;
	double bearing_to_destination;
	double distance_to_target;
	double gpsalt;
	char gpsutc[10];
	char gpsdtg[7];
	char gpsstat[5];
	char gpslat[10];
	char gpslatdir[5];
	char gpslng[11];
	char gpslngdir[5];
	double gpsvar;
	char gpsvardir[5];
	char tempchar[10];
	char usrsentence[10];
	char usrsent2;

	StrCopy(usrsentence, data.config.altsentence);
	usrsent2 = usrsentence[2];
	while (cur<length) {
		while( (cur<length) && (next<80) && (serinp[cur] != '$')  ) {
			buf[next]=serinp[cur];
			next++;
			cur++;
		}
		if (next >= 80) {
			/* Parsing error start over */
			next=0;
		}
		if ((cur<length) && (serinp[cur] == '$')) {
			cur++;
			if (buf[0]!='G' && buf[0]!='P' && buf[0]!=usrsentence[0]) {
//			if (buf[0]!='G' && buf[0]!='P') {
				/* Parsing error start over*/
				next=0;
			} else {
				buf[next]=0;
				switch (buf[2]) {
					case 'B': 
						if ((buf[3]=='W') && (buf[4]=='C')) {
							StrCopy(GPBWC,buf);
							state|=GPBWC_VALID;
						}
						break;
					case 'V':
						if ((buf[3]=='T') && (buf[4]=='G')) {
							StrCopy(GPVTG,buf);
							state|=GPVTG_VALID;
						}
						break;
					case 'R': 
						if ((buf[3]=='M') && (buf[4]=='Z')) {
							StrCopy(PGRMZ,buf);
							state|=PGRMZ_VALID;
						} else if ((buf[3]=='M') && (buf[4]=='C')) {
							StrCopy(GPRMC,buf);
							state|=GPRMC_VALID;
						} else if ((buf[3]=='M') && (buf[4]=='B')) {
							StrCopy(GPRMB,buf);
							state|=GPRMB_VALID;
						}
						break;
					case 'G': 
						if ((buf[3]=='G') && (buf[4]=='A')) {
							StrCopy(GPGGA,buf);
							state|=GPGGA_VALID;
						}
						break;
/*					case 'G':
						if ((buf[3]==usrsentence[3]) && (buf[4]==usrsentence[4])) {
							StrCopy(USRALT,buf);
							state|=USRALT_VALID;
						}
						break;
*/
					default:
						break;
				}
				next=0;
			}
		}
	}
	
	//GPS Sentence to give distance & range to target waypoint
	if ((state & GPRMB_VALID) !=0) {
		s=&GPRMB[0];
		//	MFH Getting distance to waypoint in nautical miles
		GetField(s, 10, tempchar);
		distance_to_target=StrToDbl(tempchar);
		//	MFH Getting bearing to waypoint in true degrees then  convert to mag
		GetField(s, 11, tempchar);
		bearing_to_destination=StrToDbl(tempchar)+data.input.deviation.value;
	
		if (StrLen(tempchar) > 2) {
			data.input.distance_to_target.value=distance_to_target; 
			data.input.distance_to_target.valid=VALID;
			data.input.bearing_to_destination.value=bearing_to_destination;
			data.input.bearing_to_destination.valid=VALID;
			data.application.changed = 1;
		}
	}

	//GPS Sentence to give course, speed and determine magnetic variation 
	//	 Also gives GPS Time, Status & LAT/LNG for logging
	if ((state & GPRMC_VALID) !=0) {
		Boolean retval;
		s=&GPRMC[0];
		GetField(s, 1, tempchar);
		StrCopy(gpsutc, tempchar);
		GetField(s, 2, tempchar);
		StrCopy(gpsstat, tempchar);
		GetField(s, 3, tempchar);
		StrCopy(gpslat, tempchar);
		GetField(s, 4, tempchar);
		StrCopy(gpslatdir, tempchar);
		GetField(s, 5, tempchar);
		StrCopy(gpslng, tempchar);
		GetField(s, 6, tempchar);
		StrCopy(gpslngdir, tempchar);
		// MFH GPS Speed Over Ground in knots
		GetField(s, 7, tempchar);
		groundspeed = StrToDbl(tempchar);
		GetField(s, 8, tempchar);
		true_track = StrToDbl(tempchar);
		GetField(s, 9, tempchar);
		StrCopy(gpsdtg, tempchar);
		GetField(s, 10, tempchar);
		gpsvar = StrToDbl(tempchar);
		retval=GetField(s, 11, tempchar);
		StrCopy(gpsvardir, tempchar);

		//Mag = True - Var(E) or + Var(W)
		if (StrCompare(gpsvardir, "E")==0) {
			gpsvar *= -1;
		}
		magnetic_track=true_track + gpsvar;
	
		data.input.ground_speed.value=groundspeed;
		data.input.ground_speed.valid=VALID;
		data.input.magnetic_track.value=magnetic_track;
		data.input.magnetic_track.valid=VALID;
		data.input.deviation.value=gpsvar;
		data.input.deviation.valid=VALID;

		StrCopy(data.logger.gpsutc,gpsutc);
		StrCopy(data.logger.gpsdtg,gpsdtg);
		StrCopy(data.logger.gpsstat,gpsstat);
		StrCopy(data.logger.gpslat,gpslat);
		StrCopy(data.logger.gpslatdir,gpslatdir);
		StrCopy(data.logger.gpslng,gpslng);
		StrCopy(data.logger.gpslngdir,gpslngdir);

		data.application.changed = 1;	
	}

	//Garmin Proprietary Sentence to give GPS Altitude in feet 
	if ((state & PGRMZ_VALID) !=0) {
		if (gpsver2 == 1) {
			revert++;
			if (revert == 4) {
				gpsver2 = 0;
				revert = 0;
			}
		} else {
			s=&PGRMZ[0];
			GetField(s, 1, tempchar);
			gpsalt = StrToDbl(tempchar);
	
			data.logger.gpsalt=gpsalt; 
			data.logger.pressalt=gpsalt;
			data.application.changed = 1;
		}
	}

	//NMEA183 V2 Sentence to give GPS Altitude in meters 
	if ((state & GPGGA_VALID) !=0) {
		gpsver2 = 1;
		s=&GPGGA[0];
		GetField(s, 9, tempchar);
		gpsalt = StrToDbl(tempchar);

		//Altitude in meters divided by ALTMETCONST to convert to feet.
		data.logger.gpsalt=gpsalt/ALTMETCONST; 
		data.logger.pressalt=gpsalt;
		data.application.changed = 1;
	}
	//User Defined Sentence to give GPS Altitude 
	if ((state & USRALT_VALID) !=0) {
		gpsver2 = 1;
		s=&USRALT[0];
		GetField(s, 9, tempchar);
		gpsalt = StrToDbl(tempchar);

		//Altitude in meters divided by ALTMETCONST to convert to feet.
		data.logger.gpsalt=gpsalt/ALTMETCONST; 
		data.logger.pressalt=gpsalt;
		data.application.changed = 1;
	}
	state = 0;
}

