#include "soaring.h"
#include "util.h"
#include "form.h"
#include "soarDB.h"
#include <Pilot.h>	// all the system toolbox headers
#include <SerialMgr.h>
#include <MemoryMgr.h>
#include <KeyMgr.h>
#include <SysEvtMgr.h>

ApplicationData data;
int ManualDistChg=0;
Boolean LoggingOn;
int dbcreate;

DmOpenRef config_db;
DmOpenRef logger_db;
DmOpenRef flight_db;
UInt      config_rec = 0;
UInt		cardNo=0;

static int StartApplication(void)
{
	Err error;
	int dberr;
	VoidHand record_handle;
	VoidPtr record_ptr;

	if (RxInit()) {
		return true;
	}

	MemSet(&data,sizeof(data),0); /* all fields are invalid */

	/* Configuration data */
	dberr=OpenCreateDB(&config_db, cardNo, config_db_name, creator, config_db_type, DB_RW_MODE);
	if (dberr == -1) {
		ErrDisplay("Could not create config database");
		RxClose();
		return true;
	}

	if (dberr==1) {
    
		data.config.bugfactor=1.0;
		data.config.units=NAUTICAL;
		data.config.logpolldate=TimGetSeconds();
		set_unit_constants();
 
		OpenDBAddRecord(config_db, sizeof(data.config), &data.config, true);
		OpenDBSetBackup(config_db, true);

	}
	OpenDBQueryRecord(config_db, config_rec, &record_handle, &record_ptr);
	MemMove(&data.config,record_ptr,sizeof(data.config));
	MemHandleUnlock(record_handle);

	/* Logging Data */
	dberr=OpenCreateDB(&logger_db, cardNo, logger_db_name, creator, logger_db_type, DB_RW_MODE);
	if (dberr == -1) {
		ErrDisplay("Could not create logger database");
		return true;
	}
	if (dberr==1) {
		OpenDBSetBackup(logger_db, true);
	}
	LoggingOn = false;

	dberr=OpenCreateDB(&flight_db, cardNo, flight_db_name, creator, flight_db_type, DB_RW_MODE);
	if (dberr == -1) {
		ErrDisplay("Could not create logger database");
	}
	if (dberr==1) {
		OpenDBSetBackup(flight_db, true);
	}

	FrmGotoForm(form_final_glide);  

	data.parser.parser_func = parser_standard;
	StrCopy(data.parser.name,"Standard parser\0");
  
	return 0;
}

static void StopApplication(void)
{
	Err error;
	VoidHand record_handle;
	Ptr record_ptr;

	RxClose();

	record_handle = DmGetRecord(config_db, config_rec);
	record_ptr=MemHandleLock(record_handle);
	DmWrite(record_ptr,0,&data.config,sizeof(data.config));
	MemHandleUnlock(record_handle);
	DmReleaseRecord(config_db,config_rec,true);
	DmCloseDatabase(config_db);
	DmCloseDatabase(logger_db);
	DmCloseDatabase(flight_db);

	return;
}


static Boolean ApplicationHandleEvent(EventPtr event)
{
	FormPtr frm;
	Int formId;
	Boolean handled = false;

	switch (event->eType) {
		case frmLoadEvent:
			formId = event->data.frmLoad.formID;
			frm = FrmInitForm(formId);
			FrmSetActiveForm(frm);
			MemSet(&(data.application),
				sizeof(data.application),0); /* all fields are invalid */
			data.application.form_id=formId;
			switch (formId) {
				case form_final_glide:
					FrmSetEventHandler(frm, form_final_glide_event_handler);
					data.application.display_events[0].valid=VALID;
					data.application.display_events[0].event=final_glide_event;
					data.application.display_events[1].valid=VALID;
					data.application.display_events[1].event=final_glide_event0;
					data.application.display_events[2].valid=VALID;
					data.application.display_events[2].event=final_glide_event1;
					data.application.display_events[3].valid=VALID;
					data.application.display_events[3].event=final_glide_event2;
					data.application.display_events[4].valid=VALID;
					data.application.display_events[4].event=final_glide_event3;
					data.application.display_events[5].valid=VALID;
					data.application.display_events[5].event=final_glide_event4;
					data.application.display_events[6].valid=VALID;
					data.application.display_events[6].event=final_glide_event5;
					break;
				case form_set_polar:
					FrmSetEventHandler(frm, form_set_polar_event_handler);
					break;
				case form_set_units:
					FrmSetEventHandler(frm, form_set_units_event_handler);
					break;
				case form_set_logger:
					FrmSetEventHandler(frm, form_set_logger_event_handler);
					break;
				case form_flt_info:
					FrmSetEventHandler(frm, form_flt_info_event_handler);
					break;
			}
			handled = true;
			break;
		case menuEvent:
			switch (event->data.menu.itemID) {
				case menu_final_glide:
					FrmGotoForm(form_final_glide);  
					break;
				case menu_set_polar:
					FrmGotoForm(form_set_polar);  
					break;
				case menu_set_units:
					FrmGotoForm(form_set_units);  
					break;
				case menu_set_logger:
					FrmGotoForm(form_set_logger);  
					break;
				case menu_flt_info:
					FrmGotoForm(form_flt_info);  
					break;
			}
			handled = true;
			break; 
		case nilEvent:
			if ((StrCompare(data.logger.gpsstat, "A")==0) && (data.config.logonoff == true)) {
				LoggingOn = true;
			} else {
				LoggingOn = false;
			}
			while(RxData()); 
			break;
	}
	return handled;
}

static Boolean PreprocessEvent(EventPtr event)
{
  Boolean handled=false;
  double tempdbl;

  switch (event->eType) {
	case keyDownEvent:
      switch (event->data.keyDown.chr) {
			case hard1Chr:
				tempdbl = field_get_double(form_final_glide_headwinnd);
				tempdbl += 1.0;
				data.config.headwind = tempdbl/data.config.unitsconst;
				field_set_value(form_final_glide_headwinnd, DblToStr(tempdbl,1));
				data.application.changed = 1;
				handled = true;
				break;
			case hard2Chr:
				tempdbl = field_get_double(form_final_glide_headwinnd);
				tempdbl -= 1.0;
				data.config.headwind = tempdbl/data.config.unitsconst;
				field_set_value(form_final_glide_headwinnd, DblToStr(tempdbl,1));
				data.application.changed = 1;
				handled = true;
				break;
			case hard3Chr:
				tempdbl = field_get_double(form_final_glide_airmass);
				tempdbl += data.config.airmassincr;
				data.config.airmass = tempdbl/data.config.airmassconst;
				field_set_value(form_final_glide_airmass, 
					DblToStr(tempdbl,data.config.airmassprec));
				data.application.changed = 1;
				handled=true;
				break;
			case hard4Chr:
				tempdbl = field_get_double(form_final_glide_airmass);
				tempdbl -= data.config.airmassincr;
				data.config.airmass = tempdbl/data.config.airmassconst;
				field_set_value(form_final_glide_airmass, 
					DblToStr(tempdbl,data.config.airmassprec));
				data.application.changed=1;
				handled=true;
				break;
			case pageDownChr:
				tempdbl = field_get_double(form_final_glide_dis);
				tempdbl -= 1.0;
				if ((data.input.distance_to_target.valid==VALID) && 
					(data.input.distance_to_target.value > 0)) {
					data.input.distance_to_target.value=tempdbl/data.config.unitsconst;
					field_set_value(form_final_glide_dis, DblToStr(tempdbl,0));
					ManualDistChg = 1;
				}
				data.application.changed = 1;
				handled = true;
				break;
			case pageUpChr:
				tempdbl = field_get_double(form_final_glide_dis);
				tempdbl += 1.0;
				if (data.input.distance_to_target.valid==VALID) {
					data.input.distance_to_target.value=tempdbl/data.config.unitsconst;
					field_set_value(form_final_glide_dis, DblToStr(tempdbl,0));
					ManualDistChg = 1;
				} else {
					data.input.distance_to_target.value=0.0;		
				}
				data.input.distance_to_target.valid=VALID;
				data.application.changed = 1;
				handled = true;
				break;
			case autoOffChr:
				handled = true;
				break;
			case calcChr:
				if (FrmGetActiveFormID() == form_final_glide) {
					FrmGotoForm(form_set_logger);
				}
				if (FrmGetActiveFormID() == form_set_logger) {
					FrmGotoForm(form_final_glide);
				}
				handled = true;
				break;
			case findChr:
				if (FrmGetActiveFormID() == form_final_glide) {
					FrmGotoForm(form_flt_info);
				}
				if (FrmGetActiveFormID() == form_flt_info) {
					FrmGotoForm(form_final_glide);
				}
				handled = true;
				break;
			default:
				break;
		}     
      break;
	default:
		break;
	}
	return handled;
}
    
static void EventLoop(void)
{
	EventType event;
	Word error;
	do {
		/* wakeup every half second since data arrives every second*/
		EvtGetEvent(&event,SysTicksPerSecond()/2); 
		EvtResetAutoOffTimer();
		if (! PreprocessEvent(&event)) 
			if (! SysHandleEvent(&event))
				if (! MenuHandleEvent(NULL, &event, &error))
					if (! ApplicationHandleEvent(&event))
						FrmDispatchEvent(&event);
		if ((event.eType == nilEvent) && (data.application.changed !=0)) {
			/* recalculate form */
			int x;
			data.application.changed = 0;
			x=0;
			while((x<MAX_EVENT_SZ)
				&&(data.application.display_events[x].valid==VALID)){
				data.application.display_events[x].event();
				/* make sure we handle events while we redcalculate */
				while (EvtEventAvail()) { 
				EvtGetEvent(&event,0); 
				if (! PreprocessEvent(&event)) 
					if (! SysHandleEvent(&event))
						if (! MenuHandleEvent(NULL, &event, &error))
							if (! ApplicationHandleEvent(&event))
								FrmDispatchEvent(&event);
				if (event.eType == appStopEvent)
					return;
				}
	      	x++;
			}
		}
	} while (event.eType != appStopEvent);
}

DWord PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Word error=0;

	if (cmd == sysAppLaunchCmdNormalLaunch) {
		error = StartApplication(); // Application start code
		if (error) return error;
		EventLoop(); // Event loop
		StopApplication(); // Application stop code
	}
	return error;
}
