#ifndef SOARING_H
#define SOARING_H

//#pragma pack(2)
#include <System/SysAll.h>
#include <System/StringMgr.h>
#include <System/MemoryMgr.h>
#include <UI/UIAll.h>

#define MAX_EVENT_SZ 10

#define NOT_VALID 0
#define VALID     1
#define ESTIMATED 2

//Use these to convert from knots to different units
#define MPHCONST  1.151 
#define NAUCONST  1.0 
#define METCONST  1.852

#define ALTMPHNAUCONST  1.0 
//Multiply feet by constant to get meters
#define ALTMETCONST     0.3048

#define AIRFPMCONST 101.2686
#define AIRKTSCONST 1.0
#define AIRMPSCONST 0.5144444

#define AIRFPMINCR  50.0 /* 50 feet/sec in knots */
#define AIRKTSINCR  0.5
#define AIRMPSINCR  0.25 /* 0.25 m/sec in knots */

#define STATUTE   0
#define NAUTICAL  1
#define METRIC    2

#define LOGINDX   153
#define LOGINDY   12 

#define   config_db_name "SoaringPilot_config_db\0"
#define   logger_db_name "SoaringPilot_logger_db\0"
#define   flight_db_name "SoaringPilot_flight_db\0"
#define   creator 'Soar'
#define   config_db_type 'Text'
//#define    config_db_type 'Conf'
#define   logger_db_type 'Log'
#define   flight_db_type 'Flt'

typedef void (*ParserType)(char * string, int length);
typedef void (*EventHandlerType)();

typedef struct DataEvent {
    EventHandlerType event;
    int valid;
} DataEvent;
    
typedef struct DataValue {
    double value;
    int    valid;
} DataValue;

typedef struct Polar{
    double v1;
    double w1;
    double v2;
    double w2;
    double v3;
    double w3;
    double a;
    double b;
    double c;
} Polar;  

typedef struct Parser {
    ParserType parser_func;
    char       name[32];
} Parser;    
    
typedef struct ConfigFlight {
	Polar       polar;
	double      bugfactor;
	double      headwind;
	double      airmass;
	double      safealt;
	int         units;
	double      unitsconst;
	double      airmassconst;
	double      altconst;
	char        unitstext[4];
	char        distext[3];
	char        alttext[5];
	char        airmasstext[4];
	int         airmassprec;
	double      airmassincr;
	Long        logpollintv;
	Long        logpolldate;
	double      logstartspd;
	double      logstopspd;
	Long        logstoptime;
	Boolean     logonoff;
	char			altsentence[10];
	int			altfieldnum;
} ConfigFlight;    

typedef struct InputData {
/* Usual GPS */
	DataValue  magnetic_track;
	DataValue  ground_speed;
	DataValue  distance_to_target;
	DataValue  bearing_to_destination;
	DataValue  deviation;
} InputData;

typedef struct ApplicationScreen {
    int form_id;
    DataEvent display_events[MAX_EVENT_SZ];
    int changed;
} ApplicationScreen;

typedef struct LoggerData {
	char    gpsutc[10];
	char	  gpslat[10];
	char	  gpslatdir[5];
	char    gpslng[11];
	char    gpslngdir[5];
	char    gpsstat[5];
	double    pressalt;
	double    gpsalt;
	char    gpsdtg[7];
} LoggerData;

typedef struct FlightData {
	char    startutc[7];
	char    stoputc[7];
	char    dtg[7];
	int     startidx;
	int     stopidx;
} FlightData;

typedef struct ApplicationData {
	Parser            parser;
	ConfigFlight      config;
	InputData         input;
	ApplicationScreen application;
	LoggerData        logger;
	FlightData        flight;
} ApplicationData;

extern ApplicationData data;
extern int ManualDistChg;
extern Boolean LoggingOn;
extern DmOpenRef logger_db;
extern DmOpenRef flight_db;

/* serial interface */ 
Boolean RxData();
Boolean RxInit();
void RxClose();
void TxData(char *data);

/* parser */
void parser_standard(char * serinp, int length);


/* form handler */
Boolean form_final_glide_event_handler(EventPtr event);
Boolean form_set_polar_event_handler(EventPtr event);
Boolean form_set_units_event_handler(EventPtr event);
Boolean form_set_logger_event_handler(EventPtr event);
Boolean form_flt_info_event_handler(EventPtr event);
void final_glide_event();
void final_glide_event0();
void final_glide_event1();
void final_glide_event2();
void final_glide_event3();
void final_glide_event4();
void final_glide_event5();

typedef struct IGCInfo {
	char	  pilotname[55];
	char	  glidertype[55];
	char    gliderid[25];
	char    compid[5];
	char    gpsdatum[25];
	char    compclass[25];
	char    site[55];
} IGCInfo;

#endif
