#include "soaring.h"
#include "util.h"
#include "form.h"
#include "soarDB.h"
#include <Pilot.h>	// all the system toolbox headers
#include <SerialMgr.h>
#include <MemoryMgr.h>

int atlogspd=0;
Long logstopdate;

Boolean form_set_polar_event_handler(EventPtr event)
{
	Boolean handled=false;
	char SinkText[4];
	double SinkConst;

	if (data.config.units==STATUTE) {
		StrCopy(SinkText, "fps");
		SinkConst=data.config.airmassconst/60;
	} else {
		StrCopy(SinkText, data.config.airmasstext);
		SinkConst=data.config.airmassconst;
	}

	switch (event->eType)
	{
		case frmOpenEvent:
			FrmDrawForm(FrmGetActiveForm());
			field_set_value(form_set_polar_v1,
				DblToStr(pround(data.config.polar.v1*data.config.unitsconst,0),0));
			field_set_value(form_set_polar_w1,
				DblToStr(pround(data.config.polar.w1*SinkConst,
				data.config.airmassprec+1),data.config.airmassprec+1));
			field_set_value(form_set_polar_v2,
				DblToStr(pround(data.config.polar.v2*data.config.unitsconst,0),0));
			field_set_value(form_set_polar_w2,
				DblToStr(pround(data.config.polar.w2*SinkConst,
				data.config.airmassprec+1),data.config.airmassprec+1));
			field_set_value(form_set_polar_v3,
				DblToStr(pround(data.config.polar.v3*data.config.unitsconst,0),0));
			field_set_value(form_set_polar_w3,
				DblToStr(pround(data.config.polar.w3*SinkConst,
				data.config.airmassprec+1),data.config.airmassprec+1));
			field_set_value(form_set_polar_bugfactor,
				DblToStr(data.config.bugfactor,2));
			field_set_value(form_set_safety_altitude,
				DblToStr(data.config.safealt,0));

			WinDrawChars(data.config.unitstext, 3, 142, 15);
			WinDrawChars(SinkText, 3, 142, 30);
			WinDrawChars(data.config.unitstext, 3, 142, 45);
			WinDrawChars(SinkText, 3, 142, 60);
			WinDrawChars(data.config.unitstext, 3, 142, 75);
			WinDrawChars(SinkText, 3, 142, 90);
			WinDrawChars("%  ", 3, 142, 105);
			WinDrawChars(data.config.alttext, 2, 142, 120);
			handled=true;
			break;
		case frmCloseEvent: 
		{
			double v1,v2,v3,w1,w2,w3,d;

			data.config.bugfactor = 
				field_get_double(form_set_polar_bugfactor);
			data.config.safealt = 
				field_get_double(form_set_safety_altitude);
			data.config.polar.v1 = 
				field_get_double(form_set_polar_v1)/data.config.unitsconst;
			v1=data.config.polar.v1;
			data.config.polar.w1 = 
				field_get_double(form_set_polar_w1)/SinkConst;
			w1=data.config.polar.w1/data.config.bugfactor;
			data.config.polar.v2 = 
				field_get_double(form_set_polar_v2)/data.config.unitsconst;
			v2=data.config.polar.v2;
			data.config.polar.w2 = 
				field_get_double(form_set_polar_w2)/SinkConst;
			w2=data.config.polar.w2/data.config.bugfactor;
			data.config.polar.v3 = 
				field_get_double(form_set_polar_v3)/data.config.unitsconst;
			v3=data.config.polar.v3;
			data.config.polar.w3 = 
				field_get_double(form_set_polar_w3)/SinkConst;
			w3=data.config.polar.w3/data.config.bugfactor;

			d = v1*v1*(v2-v3)+v2*v2*(v3-v1)+v3*v3*(v1-v2);
			if (d == 0.0) {
				data.config.polar.a=0;
			} else {
				data.config.polar.a=((v2-v3)*(w1-w3)+(v3-v1)*(w2-w3))/d;
			}
			d = v2-v3;
			if (d == 0.0) {
				data.config.polar.b=0;
			} else {
				data.config.polar.b = 
					(w2-w3-data.config.polar.a*(v2*v2-v3*v3))/d;
			}	 
			data.config.polar.c = 
				w3 - data.config.polar.a*v3*v3 - data.config.polar.b*v3;
			handled=false;
		}
			break;
		default:
			break;
	}
	return handled;
}

Boolean form_final_glide_event_handler(EventPtr event)
{
	Boolean handled =false;
	char MCUnitsTxt[6];
	char TempChar[6];
	int x;
	int MCMult = 1;
	int MCXVal = 15;
	int MCYVal = 38;
	int MCWidth = 2;
	switch (event->eType)
	{
		case frmOpenEvent:
			FrmDrawForm(FrmGetActiveForm());
			field_set_value(form_final_glide_headwinnd,
				DblToStr(pround(data.config.headwind*data.config.unitsconst,1),1));
			field_set_value(form_final_glide_airmass,
				DblToStr(pround(data.config.airmass*data.config.airmassconst,
				data.config.airmassprec+1), data.config.airmassprec));
			if (LoggingOn==true) {
				WinDrawInvertedChars("L", 1, LOGINDX, LOGINDY);
			} else {
				WinDrawChars("   ", 3, LOGINDX, LOGINDY);
			}
			StrCopy(TempChar, "(");
			StrCat(TempChar, data.config.airmasstext);
			StrCat(TempChar, ")");
			WinDrawChars(TempChar, 5, 1, 24);

			StrCopy(TempChar, "(");
			StrCat(TempChar, data.config.unitstext);
			StrCat(TempChar, ")");
			WinDrawChars(TempChar, 5, 28, 24);

			StrCopy(TempChar, "(");
			StrCat(TempChar, data.config.alttext);
			StrCat(TempChar, ")");
			WinDrawChars(TempChar, 4, 65, 24);

			WinDrawChars(data.config.unitstext, 3, 142, 53);
			WinDrawChars(data.config.airmasstext, 3, 142, 83);
			WinDrawChars(data.config.alttext, 2, 142, 113);
			WinDrawChars(data.config.distext, 2, 68, 129);
			WinDrawChars(data.config.unitstext, 3, 142, 129);

			StrCopy(MCUnitsTxt, "0:");
			if (data.config.units==STATUTE) {
				MCMult = 100;
				MCXVal -= 10;
				MCWidth = 4;
				StrCopy(MCUnitsTxt, "000:");
			}
			WinDrawChars(MCUnitsTxt, MCWidth, MCXVal, MCYVal);
				
			for(x=1;x<6;x++) {
				StrIToA(MCUnitsTxt, x*MCMult);
				WinDrawChars(StrCat(MCUnitsTxt,":"), MCWidth, MCXVal, MCYVal+x*15);
			}
			handled=true;
			break;
		case frmCloseEvent: 
			handled=false;
			break;
		default:
			break;
	}
	return handled;
}

Boolean form_set_units_event_handler(EventPtr event)
{
	Boolean handled=false;
		switch (event->eType)
		{
			case frmOpenEvent:
				FrmDrawForm(FrmGetActiveForm());
           	switch (data.config.units) {
	   			case STATUTE:
	     				ctl_set_value(form_set_units_statute, true);
	     				ctl_set_value(form_set_units_nautical, false);
	     				ctl_set_value(form_set_units_metric, false);
	     				break;
	   			case NAUTICAL:
	     				ctl_set_value(form_set_units_statute, false);
	     				ctl_set_value(form_set_units_nautical, true);
	     				ctl_set_value(form_set_units_metric, false);
	     				break;
	   			case METRIC:
	     				ctl_set_value(form_set_units_statute, false);
	     				ctl_set_value(form_set_units_nautical, false);
	     				ctl_set_value(form_set_units_metric, true);
	     				break;
	   			default:
	     				break;
	   		}
				handled=true;
				break;
			case frmCloseEvent: 
			{
				int oldunits=data.config.units;
				if (ctl_get_value(form_set_units_statute) == true)
					data.config.units = STATUTE;
				if (ctl_get_value(form_set_units_nautical) == true)
					data.config.units = NAUTICAL;
				if (ctl_get_value(form_set_units_metric) == true)
					data.config.units = METRIC;

				if (oldunits != data.config.units) {
					set_unit_constants();
					data.config.headwind = 0.0;
					data.config.airmass = 0.0;
					data.input.distance_to_target.value = 0.0;
					data.config.safealt = 0.0;
				}
				handled=false;
			}
				break;
			default:
				break;
		}
		return handled;
}

Boolean form_set_logger_event_handler(EventPtr event)
{
	char tempchar[30];
	char output_char[80];
	int index;
	int index2;
	int x;
	int y;
	int length;
	int altlen;
	int fltidx;
	Boolean handled=false;
	VoidHand output_hand;
	VoidPtr output_ptr;
	FlightData fltdata;
	LoggerData logdata;

	MemSet(&fltdata,sizeof(fltdata),0);

	switch (event->eType)
		{
			case frmOpenEvent:
				FrmDrawForm(FrmGetActiveForm());

				ctl_set_value(form_set_logger_onoff, data.config.logonoff);

				StrIToA(tempchar, data.config.logpollintv);
				field_set_value(form_set_logger_int, tempchar);

				field_set_value(form_set_logger_onspd,
					DblToStr(pround(data.config.logstartspd*data.config.unitsconst,0),0));
				field_set_value(form_set_logger_offspd,
					DblToStr(pround(data.config.logstopspd*data.config.unitsconst,0),0));

				StrIToA(tempchar, data.config.logstoptime);
				field_set_value(form_set_logger_offtime, tempchar);

				StrIToA(tempchar, OpenDBCountRecords(logger_db));
				field_set_value(form_set_logger_numrecs, tempchar);
				WinDrawChars(data.config.unitstext, 3, 115, 40);
				WinDrawChars(data.config.unitstext, 3, 115, 53);

				StrIToA(tempchar, OpenDBCountRecords(flight_db));
				field_set_value(form_set_logger_numflts, tempchar);

				handled=true;
				break;
			case frmCloseEvent: 
				data.config.logonoff = ctl_get_value(form_set_logger_onoff);
				data.config.logpollintv = 
					field_get_long(form_set_logger_int);
				data.config.logstartspd = 
					field_get_double(form_set_logger_onspd)/data.config.unitsconst;
				data.config.logstopspd = 
					field_get_double(form_set_logger_offspd)/data.config.unitsconst;
				data.config.logstoptime = 
					field_get_long(form_set_logger_offtime);

				handled=false;
				break;
			case ctlSelectEvent:  // A control button was pressed and released.
			   if (event->data.ctlEnter.controlID == form_set_logger_delrecsbtn) {
					OpenDBEmpty(logger_db);
					StrIToA(tempchar, OpenDBCountRecords(logger_db));
					field_set_value(form_set_logger_numrecs, tempchar);

					OpenDBEmpty(flight_db);
					StrIToA(tempchar, OpenDBCountRecords(flight_db));
					field_set_value(form_set_logger_numflts, tempchar);
					handled = 1;
				}
			   if (event->data.ctlEnter.controlID == form_set_logger_xmitrecsbtn) {
					for (fltidx = 0; fltidx < DmNumRecords(flight_db); fltidx++) { 
						OpenDBQueryRecord(flight_db, fltidx, &output_hand, &output_ptr);
						MemMove(&fltdata,output_ptr,sizeof(fltdata));
						MemHandleUnlock(output_hand);
						// Xmit Data backwards
						StrCopy(output_char, "AXXX000GPS SOARINGPILOT 0.1\r\n");
						TxData(output_char);
						StrCopy(output_char, "HFDTM100GPSDATUM:WGS-84\r\n");
						TxData(output_char);
						StrCopy(output_char, "HFDTE");
						StrCat(output_char, fltdata.dtg);
						StrCat(output_char, "\r\n");
						TxData(output_char);
/* Get the number of records in the flight DB for the outer loop.
   Then step through each flight using the UTC for the B Record and
   the start & stop indexes for the start and end of the inner loop.
*/
						for (index = fltdata.startidx; index < fltdata.stopidx; index++) { 
							OpenDBQueryRecord(logger_db, index, &output_hand, &output_ptr);
							MemMove(&logdata,output_ptr,sizeof(logdata));
							MemHandleUnlock(output_hand);
/* Output B Record */
							StrCopy(output_char, "B");
							StrCat(output_char, logdata.gpsutc);
/* Remove period from the lat */
							y=0;
							length = StrLen(logdata.gpslat);
							for (x=0;x<length;x++) {
								if (logdata.gpslat[x] != '.') {
									tempchar[y] = logdata.gpslat[x];
									y++;
								}
							}
							tempchar[y] = logdata.gpslat[x];
							StrCat(output_char, tempchar);
							StrCat(output_char, logdata.gpslatdir);
/* Remove period from the lng */
							y=0;
							length = StrLen(logdata.gpslng);
							for (x=0;x<length;x++) {
								if (logdata.gpslng[x] != '.') {
									tempchar[y] = logdata.gpslng[x];
									y++;
								}
							}
							tempchar[y] = logdata.gpslng[x];
							StrCat(output_char, tempchar);
							StrCat(output_char, logdata.gpslngdir);
							StrCat(output_char, logdata.gpsstat);
/* Pressure Altitude */
							if (logdata.pressalt < 0) {
								StrCopy(tempchar, DblToStr((logdata.pressalt * -1),0));
								StrCat(output_char, "-");
								altlen = 4;
							} else {
								StrCopy(tempchar, DblToStr(logdata.pressalt,0));
								altlen = 5;
							}
							for (index2 = 0; index2 < (altlen-StrLen(tempchar)); index2++) {
								StrCat(output_char, "0");
							}
							StrCat(output_char, tempchar);
/* GPS Altitude */
							if (logdata.gpsalt < 0) {
								StrCopy(tempchar, DblToStr((logdata.gpsalt*ALTMETCONST*-1),0));
								StrCat(output_char, "-");
								altlen = 4;
							} else {
								StrCopy(tempchar, DblToStr((logdata.gpsalt*ALTMETCONST),0));
								altlen = 5;
							}
							for (index2 = 0; index2 < (altlen-StrLen(tempchar)); index2++) {
								StrCat(output_char, "0");
							}
							StrCat(output_char, tempchar);
							StrCat(output_char, "\r\n");
							TxData(output_char);
						}
					}
					handled = 1;
				}
				break;
			default:
				break;
		}
		return handled;
}

Boolean form_flt_info_event_handler(EventPtr event)
{
	Boolean handled=false;
	char tempchar[30];
	char tempchar2[30];
	ListPtr lst;
	FormPtr frm;
	ControlPtr ctl;
	int x;
	static int nrecs;
	VoidHand flt_hand;
	VoidPtr flt_ptr;
	static char **items;
	DateTimeType strtdt, stopdt, tempdt;
	ULong startsecs, stopsecs, lengthsecs;
	FlightData fltdata;

	MemSet(&fltdata,sizeof(fltdata),0);
	switch (event->eType)
		{
			case frmOpenEvent:
				FrmDrawForm(FrmGetActiveForm());
				frm = FrmGetActiveForm();
				ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,form_flt_info_pop1));
				lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,form_flt_info_pop2));
				nrecs = OpenDBCountRecords(flight_db);
				if (nrecs > 0) {
					items = (char **) MemPtrNew(nrecs * (sizeof(char *)));
					for (x = 0; x < nrecs; x++) { 
						items[x] = (char *) MemPtrNew(5 * (sizeof(char)));
						StrIToA(tempchar, (nrecs-1) - x);
						StrCopy(items[x], tempchar);
					}
					LstSetListChoices(lst, items, nrecs);
					if (nrecs < 5) {
						LstSetHeight(lst, nrecs);
					} else {
						LstSetHeight(lst, 5);
					}
					LstSetTopItem (lst, 1);
					CtlSetLabel (ctl, "Current");
					OpenDBQueryRecord(flight_db, nrecs-1, &flt_hand, &flt_ptr);
					MemMove(&fltdata,flt_ptr,sizeof(fltdata));
					MemHandleUnlock(flt_hand);
					StrIToA(tempchar, fltdata.startidx);
					field_set_value(form_flt_info_startidx, tempchar);
					StrIToA(tempchar, fltdata.stopidx);
					field_set_value(form_flt_info_stopidx, tempchar);
/* Copy the flight dtg into the start & stop DateTime Structure */
					StringToDateOrTime(fltdata.dtg, &strtdt, 0);
					StringToDateOrTime(fltdata.dtg, &stopdt, 0);
					DateToAscii (strtdt.month, strtdt.day, strtdt.year, 
									dfMDYWithSlashes, tempchar2); 
					field_set_value(form_flt_info_dtg, tempchar2);
/* Copy the start time into the start DateTime Structure */
					field_set_value(form_flt_info_start, fltdata.startutc);
					StringToDateOrTime(fltdata.startutc, &strtdt, 1);
					startsecs=TimDateTimeToSeconds(&strtdt);
/* Copy the stop time into the stop DateTime Structure */
					field_set_value(form_flt_info_stop,fltdata.stoputc);
					if (StrCompare(fltdata.stoputc, "NT")==0) {
						StringToDateOrTime(data.logger.gpsutc, &stopdt, 1);
					} else {
						StringToDateOrTime(fltdata.stoputc, &stopdt, 1);
					}
					stopsecs=TimDateTimeToSeconds(&stopdt);
/* Calculate flt length & convert HH:MM:SS for display */
					lengthsecs=stopsecs-startsecs; 
					SecondsToDateOrTimeString(lengthsecs, tempchar, 1);
					field_set_value(form_flt_info_length,tempchar);
				} else {
					CtlSetLabel (ctl, "No Flights");
					LstSetHeight(lst, 0);
				}
				handled=true;
				break;
			case frmCloseEvent: 
			{
				if (nrecs > 0) {
					MemPtrFree(items);
					for (x = 0; x < nrecs; x++) { 
						MemPtrFree(items[x]);
					}
				}
				handled=false;
			}
				break;
			case popSelectEvent:  // A control button was pressed and released.
			   if (event->data.popSelect.controlID == form_flt_info_pop1) {
					x = (nrecs-1) - event->data.popSelect.selection;
					OpenDBQueryRecord(flight_db, x, &flt_hand, &flt_ptr);
					MemMove(&fltdata,flt_ptr,sizeof(fltdata));
					MemHandleUnlock(flt_hand);
					StrIToA(tempchar, fltdata.startidx);
					field_set_value(form_flt_info_startidx, tempchar);

					StrIToA(tempchar, fltdata.stopidx);
					field_set_value(form_flt_info_stopidx, tempchar);

/* Copy the flight dtg into the start & stop DateTime Structure */
					StringToDateOrTime(fltdata.dtg, &strtdt, 0);
					StringToDateOrTime(fltdata.dtg, &stopdt, 0);
					DateToAscii (strtdt.month, strtdt.day, strtdt.year, 
									dfMDYWithSlashes, tempchar2); 
					field_set_value(form_flt_info_dtg, tempchar2);
/* Copy the start time into the start DateTime Structure */
					field_set_value(form_flt_info_start, fltdata.startutc);
					StringToDateOrTime(fltdata.startutc, &strtdt, 1);
					startsecs=TimDateTimeToSeconds(&strtdt);
/* Copy the stop time into the stop DateTime Structure */
					field_set_value(form_flt_info_stop,fltdata.stoputc);
					StringToDateOrTime(fltdata.stoputc, &stopdt, 1);
					stopsecs=TimDateTimeToSeconds(&stopdt);
/* Calculate flt length & convert to HH:MM:SS for display */
					lengthsecs=stopsecs-startsecs; 
					SecondsToDateOrTimeString(lengthsecs, tempchar, 1);
					field_set_value(form_flt_info_length,tempchar);
				}
				break;
			default:
				break;
		}
		return handled;
}

#define MIN_SPEED 20.0
#define MAX_SPEED 200.0

static void final_glide_mc(double MC, UInt speedFieldID,
										UInt deltaAltFieldID){
	double a=data.config.polar.a;
	double b=data.config.polar.b;
	double c=data.config.polar.c;
	double h = data.config.headwind;
	double safealt = data.config.safealt;
	double altitude;
	double comp = (c +  MC - data.config.airmass)/a;
	double ls = MIN_SPEED * MIN_SPEED;
	double lv = MIN_SPEED;
	double rs = MAX_SPEED * MAX_SPEED;
	double rv = MAX_SPEED;
	double ns;
	double nv;

	if (ls < comp) {
		if (!(rs > comp)) {
			/* There is no solution to the equation in the given 
				speed range */
			field_set_value(speedFieldID,"NOV");
			field_set_value(deltaAltFieldID,"NOV");
			return;
		}
	}  else {
		if ( rs < comp ) { 
			/* swap arguments so that ls < comp and rs > comp */
			ns = ls;
			ls = rs;
			rs = ns;
			nv = lv;
			lv = rv;
			rv = nv;
		} else {
			/* There is no solution to the equation in the given 
				speed range */
			field_set_value(speedFieldID,"NOV");
			field_set_value(deltaAltFieldID,"NOV");
			return;
		}
	}	    

	/* binary search for now */
	while (((rv-lv)>1.5) || ((rv-lv)<-1.5))  {
		nv = (lv+rv)/2;
		ns = nv * nv;
		if (ns<comp) {
			ls = ns;
			lv = nv;
		} else {
			rs = ns;
			rv = nv;
		}
	}
	nv = (lv+rv)/2;

	/* Go from NM to feet */
	altitude = data.input.distance_to_target.value/(nv-h)  * 
		((a*nv*nv+b*nv+c)-data.config.airmass) * 6076.11; 
	field_set_value(speedFieldID,DblToStr(nv*data.config.unitsconst,0));
	field_set_value(deltaAltFieldID,DblToStr(altitude*data.config.altconst+safealt,0));
}

void HandleEvents(EventPtr event);

void CheckEvents(void);

void final_glide_event()
{
	char tempchar[30];
	static int firsttime;
	int index;
	int dberr;
	static int dbindex;

    /* Remark the print functions check the validity of the data */
	if (!ManualDistChg) {
		field_set_value(form_final_glide_dis,
			print_distance(data.input.distance_to_target));
	}
	ManualDistChg=0;
	field_set_value(form_final_glide_gs,
		print_horizontal_speed(data.input.ground_speed));
	field_set_value(form_final_glide_bearing,
		print_direction(data.input.bearing_to_destination));
	field_set_value(form_final_glide_track,
		print_direction(data.input.magnetic_track));
	field_set_value(form_final_glide_gpsalt,
		print_altitude(data.logger.gpsalt));

	if (LoggingOn==true) {
		WinDrawInvertedChars("L", 1, LOGINDX, LOGINDY);
	} else {
		WinDrawChars("   ", 3, LOGINDX, LOGINDY);
	}
	if (data.config.logonoff == true) {
		if (data.input.ground_speed.value >= data.config.logstartspd) {
			if (atlogspd == 0) {
				StrCopy(data.flight.startutc, data.logger.gpsutc);
				StrCopy(data.flight.stoputc, "NT");
				StrCopy(data.flight.dtg, data.logger.gpsdtg);
				dbindex = OpenDBAddRecord(flight_db, sizeof(data.flight), &data.flight, false);
				firsttime = 1;
			}
			atlogspd = 1;
			logstopdate=TimGetSeconds();
		} else {
			if (data.input.ground_speed.value < data.config.logstopspd) {
				if (atlogspd == 1) {
					if (TimGetSeconds() > (logstopdate+data.config.logstoptime)) {
						StrCopy(data.flight.stoputc, data.logger.gpsutc);
						/* Save Flight Data */
						OpenDBUpdateRecord(flight_db, sizeof(data.flight), &data.flight, dbindex);
						atlogspd = 0;
					}
				}
			} else {
				if (atlogspd == 1) {
					logstopdate=TimGetSeconds();
				}
			}
		}
		if (atlogspd == 1) {
			if (TimGetSeconds() >= (data.config.logpolldate+data.config.logpollintv)) {
				field_set_value(form_final_glide_tstfld, data.logger.gpsutc);
				// Add a new GPS update to the logger database
				index = OpenDBAddRecord(logger_db, sizeof(data.logger), &data.logger, false);
				if (firsttime) {
					data.flight.startidx = index;
					firsttime = 0;
				}
				data.flight.stopidx = index;
				data.config.logpolldate=TimGetSeconds();	// new time so we don't
												 			         // continuously loop on a
															         // GPS error
			}
		} else {
			field_set_value(form_final_glide_tstfld, "OFF    ");
		}
	}
}

void final_glide_event0()
{
    if (data.input.distance_to_target.valid == VALID) {
	final_glide_mc(0.0,form_final_glide_0_speed,
		       form_final_glide_0_alt);
    }
}

void final_glide_event1()
{
    if (data.input.distance_to_target.valid == VALID) {
	final_glide_mc(1.0,form_final_glide_1_speed,
		       form_final_glide_1_alt);
    }
}

void final_glide_event2()
{
    if (data.input.distance_to_target.valid == VALID) {
	final_glide_mc(2.0,form_final_glide_2_speed,
		       form_final_glide_2_alt);
    }
}

void final_glide_event3()
{
    if (data.input.distance_to_target.valid == VALID) {
	final_glide_mc(3.0,form_final_glide_3_speed,
		       form_final_glide_3_alt);
    }
}

void final_glide_event4()
{
    if (data.input.distance_to_target.valid == VALID) {
	final_glide_mc(4.0,form_final_glide_4_speed,
		       form_final_glide_4_alt);
    }
}

void final_glide_event5()
{
    if (data.input.distance_to_target.valid == VALID) {
	final_glide_mc(5.0,form_final_glide_5_speed,
		       form_final_glide_5_alt);
    }
}
